'use strict';

var loaderUtils = require('loader-utils');
var upath = require('upath');
var globby = require('globby');
var lodash = require('lodash');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var globby__default = /*#__PURE__*/_interopDefaultLegacy(globby);

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}
/**
 * Match tags to components.
 *
 * @param tags An array of unique tags, should be unique for casing already
 * @param componentsToMatch Array of components from your project
 */


function _async$1(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var scanComponents = _async$1(function (options, srcDir) {
  var components = [];
  var filePaths = new Set();
  var scannedPaths = [];
  var path = options.path,
      pattern = options.pattern,
      _options$ignore = options.ignore,
      ignore = _options$ignore === void 0 ? [] : _options$ignore;
  var resolvedNames = new Map();
  return _await$1(globby__default['default'](pattern, {
    cwd: path,
    ignore: ignore
  }), function (_globby) {
    var _iterator = _createForOfIteratorHelper(_globby),
        _step;

    try {
      var _loop = function _loop() {
        var _file = _step.value;
        var filePath = upath.resolve(upath.join(path, _file));
        if (scannedPaths.find(function (d) {
          return filePath.startsWith(d);
        })) return {
          v: void 0
        };
        if (filePaths.has(filePath)) return {
          v: void 0
        };
        filePaths.add(filePath);
        var fileName = upath.basename(filePath, upath.extname(filePath));
        if (fileName === 'index') fileName = upath.basename(upath.dirname(filePath), upath.extname(filePath));

        if (resolvedNames.has(fileName)) {
          // eslint-disable-next-line no-console
          console.warn("Two component files resolving to the same name `".concat(fileName, "`:\n") + "\n - ".concat(filePath) + "\n - ".concat(resolvedNames.get(fileName)));
          return {
            v: void 0
          };
        }

        resolvedNames.set(fileName, filePath);
        var pascalName = pascalCase(fileName);
        var kebabName = lodash.kebabCase(fileName);
        var shortPath = filePath.replace(srcDir, '@');
        var component = {
          filePath: filePath,
          pascalName: pascalName,
          kebabName: kebabName,
          shortPath: shortPath
        };
        var resolvedComponent = component; // allow someone to configure the component mapping

        if (options.mapComponent) resolvedComponent = options.mapComponent(component); // if a component was resolved as false, then we shouldn't push it

        if (resolvedComponent) {
          // resolve the import variable in case the user wanted to modify paths or the casing
          resolvedComponent["import"] = "import ".concat(pascalName, " from \"").concat(shortPath, "\";");
          components.push(resolvedComponent);
        }
      };

      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var _ret = _loop();

        if (_typeof(_ret) === "object") return _ret.v;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    scannedPaths.push(path);
    return components;
  });
});

var pascalCase = function pascalCase(str) {
  return lodash.upperFirst(lodash.camelCase(str));
};

function matcher(tags, componentsToMatch) {
  var components = [];
  tags.forEach(function (tag) {
    var component = lodash.first(lodash.filter(componentsToMatch, function (component) {
      return tag === component.pascalName || tag === component.kebabName;
    }));
    if (component) components.push(component);
  });
  return components.sort(function (a, b) {
    return a["import"] < b["import"] ? -1 : a["import"] > b["import"] ? 1 : 0;
  });
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var loader = _async(function (source) {
  var _this2 = this;

  var _this;

  _this = this;

  _this2.cacheable(); // bail out if we're dealing with a resource with a query
  // we only want to be dealing with the 'virtual-module' of the SFC


  if (_this2.resourceQuery) return source;
  var options = loaderUtils.getOptions(_this2);
  var tags = options.extractor.call(_this2, options); // we only need to match the tags if we have some

  return !tags || tags.length <= 0 ? source : _await(scanComponents(options, upath.resolve('./src')), function (scannedComponents) {
    // make sure cache invalidation and recompile in watch mode
    scannedComponents.forEach(function (c) {
      return _this.addDependency(c.filePath);
    });

    var generateLazyLoadImport = function generateLazyLoadImport(c) {
      if (options.vueVersion === 3) return "Lazy".concat(c.pascalName, ": defineAsyncComponent(() => import('").concat(c.shortPath.replace(';', ''), "'))");
      return "Lazy".concat(c.pascalName, ": () => import('").concat(c.shortPath.replace(';', ''), "')");
    };

    var componentsWithLazyImports = scannedComponents.map(function (c) {
      return [c, Object.assign({}, c, {
        pascalName: "Lazy".concat(c.pascalName),
        "import": generateLazyLoadImport(c),
        kebabName: "lazy-".concat(c.kebabName),
        lazy: true
      })];
    }).flat(); // the components to import

    var components = matcher(tags, componentsWithLazyImports); // only if we have components to inject

    return components.length <= 0 ? source : options.injector.call(_this2, source, components);
  });
});

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

module.exports = loader;
